@@ -1,5 +1,3 @@
-require 'pathname'
-
 class Reline::Config
   attr_reader :test_mode
 
@@ -35,6 +33,12 @@ class Reline::Config
     show-all-if-ambiguous
     show-all-if-unmodified
     visible-stats
+    show-mode-in-prompt
+    vi-cmd-mode-icon
+    vi-ins-mode-icon
+    emacs-mode-string
+    enable-bracketed-paste
+    isearch-terminators
   }
   VARIABLE_NAME_SYMBOLS = VARIABLE_NAMES.map { |v| :"#{v.tr(?-, ?_)}" }
   VARIABLE_NAME_SYMBOLS.each do |v|
@@ -52,6 +56,10 @@ class Reline::Config
     @key_actors[:emacs] = Reline::KeyActor::Emacs.new
     @key_actors[:vi_insert] = Reline::KeyActor::ViInsert.new
     @key_actors[:vi_command] = Reline::KeyActor::ViCommand.new
+    @vi_cmd_mode_icon = '(cmd)'
+    @vi_ins_mode_icon = '(ins)'
+    @emacs_mode_string = '@'
+    # https://tiswww.case.edu/php/chet/readline/readline.html#IDX25
     @history_size = -1 # unlimited
     @keyseq_timeout = 500
     @test_mode = false
@@ -159,7 +167,7 @@ class Reline::Config
 
       case line
       when /^set +([^ ]+) +([^ ]+)/i
-        var, value = $1.downcase, $2.downcase
+        var, value = $1.downcase, $2
         bind_variable(var, value)
         next
       when /\s*("#{KEYSEQ_PATTERN}+")\s*:\s*(.*)\s*$/o
@@ -209,7 +217,11 @@ class Reline::Config
   def bind_variable(name, value)
     case name
     when 'history-size'
-      @history_size = value.to_i
+      begin
+        @history_size = Integer(value)
+      rescue ArgumentError
+        @history_size = 500
+      end
     when 'bell-style'
       @bell_style =
         case value
@@ -227,7 +239,7 @@ class Reline::Config
     when 'completion-query-items'
       @completion_query_items = value.to_i
     when 'isearch-terminators'
-      @isearch_terminators = instance_eval(value)
+      @isearch_terminators = retrieve_string(value)
     when 'editing-mode'
       case value
       when 'emacs'
@@ -248,12 +260,35 @@ class Reline::Config
       end
     when 'keyseq-timeout'
       @keyseq_timeout = value.to_i
+    when 'show-mode-in-prompt'
+      case value
+      when 'off'
+        @show_mode_in_prompt = false
+      when 'on'
+        @show_mode_in_prompt = true
+      else
+        @show_mode_in_prompt = false
+      end
+    when 'vi-cmd-mode-string'
+      @vi_cmd_mode_icon = retrieve_string(value)
+    when 'vi-ins-mode-string'
+      @vi_ins_mode_icon = retrieve_string(value)
+    when 'emacs-mode-string'
+      @emacs_mode_string = retrieve_string(value)
     when *VARIABLE_NAMES then
       variable_name = :"@#{name.tr(?-, ?_)}"
       instance_variable_set(variable_name, value.nil? || value == '1' || value == 'on')
     end
   end
 
+  def retrieve_string(str)
+    if str =~ /\A"(.*)"\z/
+      parse_keyseq($1).map(&:chr).join
+    else
+      parse_keyseq(str).map(&:chr).join
+    end
+  end
+
   def bind_key(key, func_name)
     if key =~ /\A"(.*)"\z/
       keyseq = parse_keyseq($1)
